#!/bin/bash
###############################################################
################## v2/v3 scanner ##############################
# a little bash script to scan the darkweb for tor addresses
# adjust the proxy variables in /etc/lynx/lynx.cfg if needed
# than just call the script from a subdir, and let run
# you will need a file called "links.txt" containing onion
# links to start the script of (there are some popular
# addresses preloaded).
# the script runs in an endless loop until there are no more
# links to be followed or until the maximum number of runs is
# achieved. you can stop anytime with CTRL+C
# without loosing anything but the currently
# running request. Another start of the script, and it will
# pickup where it was interrupted. all of this is
# EXPERIMENTAL CODE, USE AT OWN RISK
# And use it responsibly, please. Don't dos.
# version 0.16
###############################################################
max_runs=100000 # max number of links to follow
max_links=2 # max number of links to follow per host, except the addresses in seeds.txt
counter=0 # the counter variable we use to follow the number of links
up=0 # the number of hosts found to be up (= answering with 200 to a HEAD request)
down=0 # the number of hosts found to be down
touch ./up.txt
touch ./down.txt
touch ./visited_links.txt # the list of all links we have tried
touch ./blocklist.txt # a list of addresses not to follow (empty by default)
touch ./max_visit.txt # the file to track which hosts have been visited max_times
touch ./seeds.txt # list of addresses to which max_times does not apply
touch ./all_links.txt # unsorted list of all onion links found, with duplicates
touch ./results.txt # addresses and stats for hosts found to be up
touch ./heartbeat.txt # file with status logs
echo "started new go at" >> heartbeat.txt
date >> heartbeat.txt
while true
do
	counter=$(("$counter"+1))
	up=$(wc -l up.txt | cut -f1 -d' ')
	down=$(wc -l down.txt | cut -f1 -d' ')
	sort -u links.txt | grep -E 'http:\/\/[[:alnum:]]{16,76}\.onion.*' | sort > links_new.txt && mv links_new.txt links.txt
	sort -u links.txt | grep -o -E 'http:\/\/[[:alnum:]]{16,76}\.onion' | sort -u | sort > hosts.txt
	sort -u visited_links.txt | sort > visited_links_new.txt && mv visited_links_new.txt visited_links.txt
	sort -u max_visit.txt | sort > max_visit_new.txt && mv max_visit_new.txt max_visit.txt
	sort -u down.txt | sort > down_new.txt && mv down_new.txt down.txt
	sort -u up.txt | sort > up_new.txt && mv up_new.txt up.txt
	comm -23 links.txt visited_links.txt > links_new.txt && mv links_new.txt links.txt
	comm -23 hosts.txt blocklist.txt > hosts_new.txt && mv hosts_new.txt hosts.txt
	comm -23 hosts.txt down.txt > hosts_new.txt && mv hosts_new.txt hosts.txt
	comm -23 hosts.txt max_visit.txt > hosts_new.txt && mv hosts_new.txt hosts.txt
	visited_link_count=$(wc -l visited_links.txt | cut -f1 -d ' ')
	link_count=$(wc -l links.txt | cut -f1 -d ' ')
	service_count=$(wc -l hosts.txt | cut -f1 -d' ')
	if [ -z $service_count ]; then
		echo "no hosts left, we are done\n\r"
		exit 0
	fi
	visited_service_count=$(grep -E -o 'http:\/\/[[:alnum:]]{16,76}\.onion' visited_links.txt | sort -u | wc -l)
	echo -e "We have tried to visit $visited_link_count links on $visited_service_count hosts, found $up hosts up, and $down hosts down. We still have $link_count unique links on $service_count hosts\n\r"
	service=$(cat "hosts.txt" "up.txt" | sort -R | head -n 1)
	service_dir=$(echo "$service" | cut -c8- | rev | cut -c7- | rev)
	echo -e "checking if $service is up or not\n\r"
	if lynx -head -dump "$service" 2>&1 | grep -w "200" ; then
		if [ ! -d "./hosts/$service_dir" ]; then
			mkdir "hosts/$service_dir"
			touch "hosts/$service_dir/links.txt" 
		fi
    		echo "$service is up"
		echo "$service" >> up.txt
		link=$(grep "^$service" links.txt | sort -R | head -n 1)
		if [ -z "$link" ]; then
			echo "no specific links for $service, using onion address"
			link="$service"
		fi
		echo -e "trying to get $link in run number $counter,\n\r"
		echo "$link" >> visited_links.txt
		lynx -dump -listonly "$link" | grep -E 'http:\/\/[[:alnum:]]{16,76}\.onion.*' | grep -E -i -v '.*\.jpg$|.*\.gif$|.*\.png$|.*\.pdf$|.*\.mp3$|.*\.m3u$|.*\.avi$|.*\.jpeg$|.*\.bmp$|.*\.mkv$' | cut -f2- -d '.' | cut -c2- >> "hosts/$service_dir/links.txt"
		echo "$link" >> "hosts/$service_dir/visited_links.txt"
		link_count=$(wc -l "hosts/$service_dir/visited_links.txt" | cut -f1 -d' ')
		seed=$(grep "$service" seeds.txt | wc -l)
		if [ "$link_count" -ge $max_links ] && [ -z $seed ]; then
			echo "$service" >> max_visit.txt
		fi
		echo "$(<hosts/$service_dir/links.txt )" >> ./links.txt
		echo "$(<hosts/$service_dir/links.txt )" >> ./all_links.txt
		to_internal_links=$(sort -u "hosts/$service_dir/links.txt" | grep "$service" | wc -l)
		to_internal_links_all=$(grep "$service" "hosts/$service_dir/links.txt" | wc -l)
		to_external_links=$(sort -u "hosts/$service_dir/links.txt" | grep -v "$service" | wc -l)
		from_external_links_total=$(grep "$service" "all_links.txt" | wc -l)
		from_external_links=$(($from_external_links_total - $to_internal_links_all))
		echo "$service|$to_internal_links|$to_external_links|$from_external_links"
		echo "$service|$to_internal_links|$to_external_links|$from_external_links" >> results.txt
	else
		echo "$service is down"
		echo "$service" >> down.txt
	fi
	date_string=$(date)
	if ((counter%5 == 0)); then
		echo -e "$date_string, we have tried to visit $visited_link_count links on $visited_service_count hosts, found $up hosts up, and $down hosts down. We still have $link_count unique links on $service_count hosts" >> "heartbeat.txt"
	fi
	if [ "$counter" -ge $max_runs ]; then
		echo "ran $count times, reached the maximum defined number of runs, exiting now"
		echo -e "$date_string, we have tried to visit $visited_link_count links on $visited_service_count hosts, found $up hosts up, and $down hosts down. We still have $link_count unique links on $service_count hosts" >> "heartbeat.txt"
		echo -e "$date_string, we have reached the maximum number of runs, exiting" >> "heartbeat.txt"
		exit 0
	fi
done

